class PayWithTransferInit {
  bool? success;
  String? message;
  Request? request;

  PayWithTransferInit({this.success, this.message, this.request});

  PayWithTransferInit.fromJson(Map<String, dynamic> json) {
    success = json['success'];
    message = json['message'];
    request =
    json['request'] != null ? Request.fromJson(json['request']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['success'] = success;
    data['message'] = message;
    if (request != null) {
      data['request'] = request!.toJson();
    }
    return data;
  }
}

class Request {
  String? id;
  BankAccount? bankAccount;
  String? status;
  String? wsChannel;
  PayWithTransferRequestData? meta;
  List<Transaction>? transaction;
  List<RejectedTransaction>? rejectedTransaction;

  Request(
      {this.id,
        this.bankAccount,
        this.status,
        this.wsChannel,
        this.meta,
        this.transaction,
        this.rejectedTransaction});

  Request.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    bankAccount = json['bankAccount'] != null
        ? BankAccount.fromJson(json['bankAccount'])
        : null;
    status = json['status'];
    wsChannel = json['wsChannel'];
    if (json['transaction'] != null) {
      transaction = <Transaction>[];
      json['transaction'].forEach((v) {
        transaction!.add(Transaction.fromJson(v));
      });
    }
    meta = json['meta'] != null
        ? PayWithTransferRequestData.fromJson(json['meta'])
        : null;

    if (json['rejectedTransaction'] != null) {
      rejectedTransaction = <RejectedTransaction>[];
      json['rejectedTransaction'].forEach((v) {
        rejectedTransaction!.add(RejectedTransaction.fromJson(v));
      });
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    if (bankAccount != null) {
      data['bankAccount'] = bankAccount!.toJson();
    }
    data['status'] = status;
    data['wsChannel'] = wsChannel;
    if (transaction != null) {
      data['transaction'] = transaction!.map((v) => v.toJson()).toList();
    }
    if (rejectedTransaction != null) {
      data['rejectedTransaction'] =
          rejectedTransaction!.map((v) => v.toJson()).toList();
    }
    if (meta != null) {
      data['meta'] = meta!.toJson();
    }
    return data;
  }
}

class BankAccount {
  String? accountName;
  String? accountNumber;
  String? bankName;
  String? expiresAt;

  BankAccount(
      {this.accountName, this.accountNumber, this.bankName, this.expiresAt});

  BankAccount.fromJson(Map<String, dynamic> json) {
    accountName = json['accountName'];
    accountNumber = json['accountNumber'];
    bankName = json['bankName'];
    expiresAt = json['expiresAt'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = Map<String, dynamic>();
    data['accountName'] = accountName;
    data['accountNumber'] = accountNumber;
    data['bankName'] = bankName;
    data['expiresAt'] = expiresAt;
    return data;
  }
}

class Transaction {
  String? id;
  String? reference;
  String? amountPaid;
  String? fees;
  String? currency;
  String? paidAt;
  String? createdAt;
  Sender? sender;

  Transaction(
      {this.id,
        this.reference,
        this.amountPaid,
        this.fees,
        this.currency,
        this.paidAt,
        this.createdAt,
        this.sender});

  Transaction.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    reference = json['reference'];
    amountPaid = json['amountPaid'];
    fees = json['fees'];
    currency = json['currency'];
    paidAt = json['paidAt'];
    createdAt = json['createdAt'];
    sender =
    json['sender'] != null ? Sender.fromJson(json['sender']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = Map<String, dynamic>();
    data['id'] = id;
    data['reference'] = reference;
    data['amountPaid'] = amountPaid;
    data['fees'] = fees;
    data['currency'] = currency;
    data['paidAt'] = paidAt;
    data['createdAt'] = createdAt;
    if (sender != null) {
      data['sender'] = sender!.toJson();
    }
    return data;
  }
}

class RejectedTransaction {
  String? id;
  String? reference;
  String? amountPaid;
  String? fees;
  String? currency;
  String? rejectedAt;
  String? createdAt;
  Sender? sender;
  Reason? reason;

  RejectedTransaction(
      {this.id,
        this.reference,
        this.amountPaid,
        this.fees,
        this.currency,
        this.rejectedAt,
        this.createdAt,
        this.sender,
        this.reason});

  RejectedTransaction.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    reference = json['reference'];
    amountPaid = json['amountPaid'];
    fees = json['fees'];
    currency = json['currency'];
    rejectedAt = json['rejectedAt'];
    createdAt = json['createdAt'];
    sender =
    json['sender'] != null ? Sender.fromJson(json['sender']) : null;
    reason =
    json['reason'] != null ? Reason.fromJson(json['reason']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['reference'] = reference;
    data['amountPaid'] = amountPaid;
    data['fees'] = fees;
    data['currency'] = currency;
    data['rejectedAt'] = rejectedAt;
    data['createdAt'] = createdAt;
    if (sender != null) {
      data['sender'] = sender!.toJson();
    }
    if (reason != null) {
      data['reason'] = reason!.toJson();
    }
    return data;
  }
}

class Sender {
  String? bankName;
  String ?accountName;

  Sender({this.bankName, this.accountName});

  Sender.fromJson(Map<String, dynamic> json) {
    bankName = json['bankName'];
    accountName = json['accountName'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['bankName'] = bankName;
    data['accountName'] = accountName;
    return data;
  }
}

class Reason {
  String? code;
  String? message;

  Reason({this.code, this.message});

  Reason.fromJson(Map<String, dynamic> json) {
    code = json['code'];
    message = json['message'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['code'] = code;
    data['message'] = message;
    return data;
  }
}

class PayWithTransferRequestData {
  int? amount;
  String? currency;
  String? email;
  String? reference;
  String ?accountName;

  PayWithTransferRequestData(
      {this.amount,
        this.currency,
        this.email,
        this.reference,
        this.accountName});

  PayWithTransferRequestData.fromJson(Map<String, dynamic> json) {
    amount = json['amount'];
    currency = json['currency'];
    email = json['email'];
    reference = json['reference'];
    accountName = json['accountName'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['amount'] = amount;
    data['currency'] = currency;
    data['email'] = email;
    data['reference'] = reference;
    data['accountName'] = accountName;
    return data;
  }
}

