import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:lenco_pay_with_transfer_widget/src/datamodel.dart';
import 'package:lenco_pay_with_transfer_widget/src/request.dart';
import 'package:lenco_pay_with_transfer_widget/src/searching_transfer.dart';

import 'package:lenco_pay_with_transfer_widget/src/timer/countdown_timer_controller.dart';
import 'package:lenco_pay_with_transfer_widget/src/timer/flutter_countdown_timer.dart';
import 'package:lenco_pay_with_transfer_widget/src/transaction_success.dart';
import 'package:lenco_pay_with_transfer_widget/src/transfer_rejected.dart';
import 'package:lenco_pay_with_transfer_widget/src/utility.dart';
import 'package:socket_io_client/socket_io_client.dart';
import 'package:shared_preferences/shared_preferences.dart';


import 'timer/current_remaining_time.dart';


class LencoPayWithTransfer extends StatefulWidget {
  final  num amount;
  final  String currency;//(optional)
  final  String email ;
  final  String reference;  //# unique (No space). Only ".", "_", and "-" are allowed
  final  String? accountName;
  final  String publicKey;
  final  Function(String) onErrorMessage;
  final  Function(Transaction) onSuccessful;
  final  Function(RejectedTransaction) onRejected;
  const LencoPayWithTransfer({Key? key, required this.amount, this.currency='NGN', required this.email, required this.reference, this.accountName, required this.publicKey, required this.onErrorMessage, required this.onSuccessful, required this.onRejected}) : super(key: key);


  @override
  _LencoPayWithTransferState createState() => _LencoPayWithTransferState();
}

class _LencoPayWithTransferState extends State<LencoPayWithTransfer> {
   TextStyle titleStyle=const TextStyle(fontSize: 20,color:Color(0xff1f1f30),fontWeight: FontWeight.bold);
   TextStyle textSubTitle=const TextStyle(color: Color(0xff41415A), fontSize: 20);
   String showMessage='Please wait...';
   //Request? _request;


   socketHandler(String token)async{


     Socket socket = io(WSURL,
         OptionBuilder()
             .setTransports(['websocket'])
            // .disableAutoConnect()
             .build());
     socket.on('pwt.transaction.successful', (data){
      Request dataValue=Request.fromJson(json.decode(json.encode(data)));
      // _request=dataValue;
       widget.onSuccessful(dataValue.transaction!.last);
       timer.cancel();
       controller.disposeTimer();
       Navigator.of(context).pop();
       Navigator.push(
       context,
       MaterialPageRoute(builder: (context) => TransactionSuccessful(close: () {



       },

       )),
     );
       setState(() {

       });


     });
     socket.on('pwt.transaction.rejected', (data){
        Request dataValue=Request.fromJson(json.decode(json.encode(data)));
      // _request=dataValue;
       widget.onRejected(dataValue.rejectedTransaction!.last);
       timer.cancel();
       controller.disposeTimer();
       Navigator.push(
           context,
           MaterialPageRoute(builder: (context) => TransferRejected(
         reason:dataValue.rejectedTransaction!.last.reason!.message! ,
         showAccountNumber: () {
         //  timer.cancel();
           _initializeAgain();


         },

       )));
       setState(() {

       });


     });
     socket.on('subscribed', (data){


     });



     socket.onConnect((data) {
        socket.emit('subscribe', token);
       socket.onDisconnect((_) =>(){});

     });

     socket.onConnectError((data) {


     });











   }
    onTimerEnd(){

     Navigator.push(
       context,
       MaterialPageRoute(builder: (context) => SearchingTransfer(showAccountNumber: () {
         timer.cancel();
         _initialize();
         setState(() {

         });

       },

       )),
     );
   }


  // late CountdownTimerController controller;
   late CountdownTimerController controller;
    PayWithTransferInit? init;
    String? _requestId;

   _initialize()async{
     SharedPreferences prefs = await SharedPreferences.getInstance();

     init=null;
     setState(() {

     });
     try{
       init=await LencoRequest(publicKey: widget.publicKey).initiate(amount: widget.amount, email: widget.email, reference: widget.reference,currency: widget.currency,accountName: widget.accountName,
       dioError: (error){
         widget.onErrorMessage('Error: ${error.response}');
       });

       if(init!.success!){
         _requestId=init!.request!.id!;
         prefs.setString('_requestId',_requestId!);
         controller=CountdownTimerController(endTime: DateTime.parse(init!.request!.bankAccount!.expiresAt!).millisecondsSinceEpoch,onEnd: onTimerEnd);
         controller.start();

         socketHandler(init!.request!.wsChannel!);
         timer = Timer.periodic(const Duration(seconds: 20), (Timer t) => _checkStatus(_requestId!));
          setState(() {});
       }
       else{
         widget.onErrorMessage('Initialization: ${init!.message}');
         showMessage=init!.message!;
         setState(() {

         });
       }
     }
     catch(e,stacktrace){
      // print(stacktrace);
      // widget.onErrorMessage('Initialization: $e');
      //timer.cancel();

      // controller.disposeTimer();


     }


   }

   _initializeAgain()async{
     init=null;
     setState(() {

     });
     try{
       init=await LencoRequest(publicKey: widget.publicKey).re_initiate(requestId: _requestId!,
           dioError: (error){
             widget.onErrorMessage('Error: ${error.response}');
           }
       );

       if(init!.success!){
         controller=CountdownTimerController(endTime: DateTime.parse(init!.request!.bankAccount!.expiresAt!).millisecondsSinceEpoch,onEnd: onTimerEnd);
         controller.start();

         socketHandler(init!.request!.wsChannel!);
         timer = Timer.periodic(const Duration(seconds: 10), (Timer t) => _checkStatus(_requestId!));
         setState(() {});
       }
       else{
         widget.onErrorMessage('Initialization: ${init!.message}');
         showMessage=init!.message!;
         setState(() {

         });
       }
     }
     catch(e,stacktrace){
      // print(stacktrace);
      //   widget.onErrorMessage('Initialization: $e');
      // timer.cancel();
       //controller.disposeTimer();

     }


   }

   late Timer timer;
        _checkStatus(String requestId)async{
           try{
           var v=await LencoRequest(publicKey: widget.publicKey).status(requestId:requestId,
               dioError: (error){
                 widget.onErrorMessage('Error: ${error.response}');
               });

            if(v.success!){
                 if(v.request!.status=='successful'){
                  timer.cancel();
                  controller.disposeTimer();
                  widget.onSuccessful(v.request!.transaction!.last);
                  Navigator.of(context).pop();


                  if(mounted){
                    setState(() {

                    });
                  }
                  Navigator.push(
                    context,
                    MaterialPageRoute(builder: (context) => TransactionSuccessful(close: () {

                      setState(() {

                      });

                    },

                    )),
                  );
                }
                else if(v.request!.status=='payment-rejected'){
                  timer.cancel();
                  controller.disposeTimer();

                  widget.onRejected(v.request!.rejectedTransaction!.last);
                  if(mounted){
                    setState(() {

                    });
                  }
                  Navigator.push(
                    context,
                    MaterialPageRoute(builder: (context) => TransferRejected(
                      reason:v.request!.rejectedTransaction!.last.reason!.message! ,
                      showAccountNumber: () {
                        _initializeAgain();


                    },

                    )),
                  );

                }

            }
            else{
              widget.onErrorMessage('Status: ${init!.message}');
              showMessage=v.message!;
              setState(() {

              });
            }
          }
          catch(e,stacktrace){
          //  widget.onErrorMessage('Initialization: $e');
            //timer.cancel();
            //controller.disposeTimer();
            //print(stacktrace);

          }
        }

   @override
   void dispose() {
    // timer.cancel();
     //controller.disposeTimer();
     super.dispose();
   }
   _init()async{

     SharedPreferences prefs = await SharedPreferences.getInstance();
     String previousRef=prefs.getString('key')??'';

     if(widget.reference==previousRef){
       _requestId=prefs.getString('_requestId');
       _initializeAgain();

     }
     else{
       _initialize();

      prefs.setString('key',widget.reference);

     }


   }

   @override
   void initState() {
    super.initState();
    _init();


    }

    Widget _bodyWidget(){
      if (init!=null) {
        if(init!.success!){
          return Center(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(

                mainAxisAlignment: MainAxisAlignment.center,
                mainAxisSize: MainAxisSize.min,
                children:  <Widget>[
                  Padding(
                    padding: const EdgeInsets.only(top:8.0),
                    child: Align(alignment: Alignment.centerRight, child: IconButton(icon: const Icon(Icons.close), onPressed: () {
                      timer.cancel();
                      controller.disposeTimer();
                      Navigator.pop(context);
                    },)),
                  ),

                  Padding(
                    padding: const EdgeInsets.only(top:8.0),
                    child: Divider(height: 3, color: const Color(0xffB1BAFF).withOpacity(0.3)),
                  ),
                  Center(
                    child: Padding(
                      padding: const EdgeInsets.only(top:8.0),
                      child: Text('Transfer',style: textSubTitle,),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top: 8.0),
                    child: FormatAmount(widget.currency,widget.amount),
                  ),
                  Center(
                    child: Padding(
                      padding: const EdgeInsets.only(top:8.0),
                      child: Text('To',style: textSubTitle,),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top:8.0),
                    child: Container(
                      width: MediaQuery.of(context).size.width-40,
                      height: 250,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey.withOpacity(0.3),),
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Text(init!.request!.bankAccount!.bankName!,style: const TextStyle(color: Color(0xff41415A), fontSize: 20,fontWeight: FontWeight.w500),),
                          Padding(
                            padding: const EdgeInsets.only(top:0.0),
                            child: InkWell(
                              onTap: ()=> Clipboard.setData( ClipboardData(text: init!.request!.bankAccount!.accountNumber!)),

                              child: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  Text(init!.request!.bankAccount!.accountNumber!,style: const TextStyle(color: Color(0xff41415A), fontSize: 20,fontWeight: FontWeight.bold),),
                                  IconButton(onPressed: ()=>Clipboard.setData( ClipboardData(text: init!.request!.bankAccount!.accountNumber!)), icon: const Icon(Icons.copy)),
                                ],
                              ),
                            ),
                          ),
                          Text(init!.request!.bankAccount!.accountName!,style: const TextStyle(color: Color(0xff41415A), fontSize: 20,fontWeight: FontWeight.w500),),
                          const Padding(
                            padding: EdgeInsets.all(10),
                            child: Text('Use this account number for this transaction only. Account expires in ',textAlign: TextAlign.center, style: TextStyle( color: Colors.grey, fontSize: 14,),),
                          ),
                          //https://github.com/wuweijian1997/FlutterCountdownTimer
                          CountdownTimer(
                              onEnd: ()=>{},
                              controller: controller,
                              widgetBuilder: (context,CurrentRemainingTime? time) {
                                if (time == null) {
                                  return const Text('Time Up');
                                }
                                return Text(
                                  // 'days: [ ${time.days} ], hours: [ ${time.hours} ], min: [ ${time.min} ], sec: [ ${time.sec} ]');
                                  '${time.min ?? '0'} : ${time.sec}',style: const TextStyle(fontSize: 19,fontWeight: FontWeight.bold),);

                              }

                          ),



                        ],
                      ),

                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top:20.0),
                    child: Container(
                      width: MediaQuery.of(context).size.width-40,
                      height: 50,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey.withOpacity(0.1),),
                      child:  Center(child: Text('Waiting for your transfer',style: titleStyle.copyWith(fontSize: 14),)),
                    ),
                  ),],
              ),
            ),
          );

        }
        else{
          return Padding(
            padding: const EdgeInsets.all(8.0),
            child: Center(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                  children: [
                  Padding(
                    padding: const EdgeInsets.only(top:8.0),
                    child: Align(alignment: Alignment.centerRight, child: IconButton(icon: const Icon(Icons.close), onPressed: () {

                      Navigator.pop(context);
                    },)),
                  ),
                  Card(
                    margin:const EdgeInsets.all(16) ,
                    color:Colors.grey.withOpacity(0.3) ,
                    child:   Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Text(showMessage,style: TextStyle(fontSize:18 ,color: Color(0xff41415A)),),
                    ),

                  ),
                ],
              ),
            ),
          );
        }
      } else {
        return Padding(
          padding: const EdgeInsets.all(8.0),
          child: Center(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                Padding(
                  padding: const EdgeInsets.only(top:8.0),
                  child: Align(alignment: Alignment.centerRight, child: IconButton(icon: const Icon(Icons.close), onPressed: () {

                    Navigator.pop(context);
                  },)),
                ),
                Card(
                  margin:const EdgeInsets.all(16) ,
                  color:Colors.grey.withOpacity(0.3) ,
                  child:   Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Text(showMessage,style: TextStyle(fontSize:18 ,color: Color(0xff41415A)),),
                  ),

                ),
              ],
            ),
          ),
        );
      }
    }
  @override
  Widget build(BuildContext context) {

   return Scaffold(
     body: SafeArea(child: _bodyWidget()),
   );
  }
}
